@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use vars qw( $VERSION );
 
-$VERSION = '0.17';
+$VERSION = '0.21';
 
 use base qw(
     Parse::HTTP::UserAgent::Base::IS
@@ -114,8 +114,10 @@ sub _do_parse {
         my $pname  = shift @{ $rv };
         my $method = '_parse_' . $pname;
         my $rvx    = $self->$method( @{ $rv } );
-        $self->[UA_PARSER] = $pname;
-        return $rvx;
+        if ( $rvx ) {
+            $self->[UA_PARSER] = $pname;
+            return $rvx;
+        }
     }
 
     return $self->_extended_probe($m, $t, $e, $c, @o) if $self->[IS_EXTENDED];
@@ -170,6 +172,9 @@ sub _extended_probe {
     return if $self->_is_netscape( @args ) && $self->_parse_netscape( @args );
     return if $self->_is_docomo(   @args ) && $self->_parse_docomo(   @args );
     return if $self->_is_generic(  @args );
+    return if $self->_is_emacs(    @args ) && $self->_parse_emacs(    @args );
+    return if $self->_is_moz_only( @args ) && $self->_parse_moz_only( @args );
+    return if $self->_is_hotjava(  @args ) && $self->_parse_hotjava(  @args );
 
     $self->[UA_UNKNOWN] = 1;
     return;
@@ -182,8 +187,9 @@ sub _object_ids {
 sub _numify {
     my $self = shift;
     my $v    = shift || return 0;
-    $v    =~ s{
+    $v    =~ s{(
                 pre      |
+                rel      |
                 alpha    |
                 beta     |
                 \-stable |
@@ -191,7 +197,14 @@ sub _numify {
                 [ab]\d+  |
                 a\-XXXX  |
                 \+
-                }{}xmsig;
+               )}{}xmsig;
+
+    if ( INSIDE_VERBOSE_TEST ) {
+        if ( $1 ) {
+            Test::More::diag("[DEBUG] _numify: removed '$1' from version string");
+        }
+    }
+
     # Gecko revisions like: "20080915000512" will cause an
     #   integer overflow warning. use bigint?
     local $SIG{__WARN__} = sub {
@@ -201,7 +214,27 @@ sub _numify {
     # if version::vpp is used it'll identify 420 as a v-string
     # add a floating point to fool it
     $v .= q{.0} if index($v, q{.}) == NO_IMATCH;
-    my $rv = version->new("$v")->numify;
+    my $rv;
+    eval {
+        $rv = version->new("$v")->numify; 1
+    } or do {
+        my $error = $@ || '[unknown error while parsing version]';
+        if ( INSIDE_UNIT_TEST ) {
+            chomp $error;
+            if ( INSIDE_VERBOSE_TEST ) {
+                Test::More::diag( "[FATAL] _numify: version said: $error" );
+                Test::More::diag(
+                    sprintf "[FATAL] _numify: UA with bogus version (%s) is: %s",
+                                $v, $self->[UA_STRING]
+                );
+                Test::More::diag( '[FATAL] _numify: ' . $self->dumper );
+            }
+            die $error;
+        }
+        else {
+            die $error;
+        }
+    };
     return $rv;
 }
 
@@ -243,8 +276,8 @@ Parse::HTTP::UserAgent - Parser for the User Agent string
 
 =head1 DESCRIPTION
 
-This document describes version C<0.17> of C<Parse::HTTP::UserAgent>
-released on C<8 October 2009>.
+This document describes version C<0.21> of C<Parse::HTTP::UserAgent>
+released on C<19 October 2011>.
 
 Quoting L<http://www.webaim.org/blog/user-agent-string-history/>:
 
@@ -394,12 +427,12 @@ Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2009 Burak Gursoy. All rights reserved.
+Copyright 2009 - 2011 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.10.0 or, 
+it under the same terms as Perl itself, either Perl version 5.12.3 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
