@@ -8,7 +8,7 @@ use constant ERROR_MAXTHON_MSIE    => 'Unable to extract MSIE from Maxthon UA-st
 use constant OPERA9                => 9;
 use constant OPERA_TK_LENGTH       => 5;
 
-$VERSION = '0.20';
+$VERSION = '0.21';
 
 sub _extract_dotnet {
     my($self, @args) = @_;
@@ -20,9 +20,11 @@ sub _extract_dotnet {
             push @dotnet, $match[0];
             next;
         }
-        if ( $e =~ RE_WINDOWS_OS && $1 ne '64' ) {
-            $self->[UA_OS] = $e;
-            next;
+        if ( $e =~ RE_WINDOWS_OS ) {
+            if ( $1 && $1 ne '64' ) {
+                $self->[UA_OS] = $e;
+                next;
+            }
         }
         push @extras, $e;
     }
@@ -151,7 +153,9 @@ sub _parse_safari {
                             ? pop   @{ $thing }
                             : shift @{ $thing }
                             ;
-    $self->[UA_DEVICE]      = shift @{$thing} if $thing->[0] eq 'iPhone';
+    if ( $thing->[0] && $thing->[0] eq 'iPhone' ) {
+        $self->[UA_DEVICE]  = shift @{$thing};
+    }
     $self->[UA_EXTRAS]      = [ @{$thing}, @others ];
 
     if ( length($self->[UA_OS]) == 1 ) {
@@ -535,8 +539,8 @@ Parse::HTTP::UserAgent::Base::Parsers - Base class
 
 =head1 DESCRIPTION
 
-This document describes version C<0.20> of C<Parse::HTTP::UserAgent::Base::Parsers>
-released on C<27 October 2009>.
+This document describes version C<0.21> of C<Parse::HTTP::UserAgent::Base::Parsers>
+released on C<19 October 2011>.
 
 Internal module.
 
@@ -550,12 +554,12 @@ Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2009 Burak Gursoy. All rights reserved.
+Copyright 2009 - 2011 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.10.0 or, 
+it under the same terms as Perl itself, either Perl version 5.12.3 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
