@@ -1,8 +1,6 @@
-BEGIN { $INC{$_} = 1 for qw(Parse/HTTP/UserAgent.pm Parse/HTTP/UserAgent/Constants.pm Parse/HTTP/UserAgent/Base/Accessors.pm Parse/HTTP/UserAgent/Base/Dumper.pm Parse/HTTP/UserAgent/Base/IS.pm Parse/HTTP/UserAgent/Base/Parsers.pm); }
+BEGIN { $INC{$_} = 1 for qw(Parse/HTTP/UserAgent.pm Parse/HTTP/UserAgent/Base/Accessors.pm Parse/HTTP/UserAgent/Base/Dumper.pm Parse/HTTP/UserAgent/Base/IS.pm Parse/HTTP/UserAgent/Base/Parsers.pm Parse/HTTP/UserAgent/Constants.pm); }
 package Parse::HTTP::UserAgent;
 sub ________monolith {}
-package Parse::HTTP::UserAgent::Constants;
-sub ________monolith {}
 package Parse::HTTP::UserAgent::Base::Accessors;
 sub ________monolith {}
 package Parse::HTTP::UserAgent::Base::Dumper;
@@ -12,11 +10,13 @@ sub ________monolith {}
 package Parse::HTTP::UserAgent::Base::Parsers;
 sub ________monolith {}
 package Parse::HTTP::UserAgent::Constants;
+sub ________monolith {}
+package Parse::HTTP::UserAgent::Constants;
 use strict;
 use warnings;
 use vars qw( $VERSION $OID @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS );
 
-$VERSION = '0.17';
+$VERSION = '0.21';
 
 use constant MINUS_ONE           => -1;
 use constant NO_IMATCH           => -1; # for index()
@@ -53,6 +53,8 @@ use constant TK_NAME             => 0;
 use constant TK_ORIGINAL_VERSION => 1;
 use constant TK_VERSION          => 2;
 
+use constant INSIDE_UNIT_TEST    => $ENV{PARSE_HTTP_USERAGENT_TEST_SUITE};
+use constant INSIDE_VERBOSE_TEST => INSIDE_UNIT_TEST && $ENV{HARNESS_IS_VERBOSE};
 use constant RE_FIREFOX_NAMES    => qr{Firefox|Iceweasel|Firebird|Phoenix }xms;
 use constant RE_DOTNET           => qr{ \A [.]NET \s+ CLR \s+ (.+?) \z    }xms;
 use constant RE_WINDOWS_OS       => qr{ \A Win(dows|NT|[0-9]+)?           }xmsi;
@@ -84,6 +86,7 @@ use constant LIST_ROBOTS         => qw(
     Googlebot
     Baiduspider+
     msnbot
+    bingbot
 ), 'Yahoo! Slurp';
 
 use base qw( Exporter );
@@ -150,6 +153,8 @@ BEGIN {
         etc => [qw(
             NO_IMATCH
             LAST_ELEMENT
+            INSIDE_UNIT_TEST
+            INSIDE_VERBOSE_TEST
         )],
     );
 
@@ -167,7 +172,7 @@ use constant ERROR_MAXTHON_MSIE    => 'Unable to extract MSIE from Maxthon UA-st
 use constant OPERA9                => 9;
 use constant OPERA_TK_LENGTH       => 5;
 
-$VERSION = '0.17';
+$VERSION = '0.21';
 
 sub _extract_dotnet {
     my($self, @args) = @_;
@@ -179,9 +184,11 @@ sub _extract_dotnet {
             push @dotnet, $match[0];
             next;
         }
-        if ( $e =~ RE_WINDOWS_OS && $1 ne '64' ) {
-            $self->[UA_OS] = $e;
-            next;
+        if ( $e =~ RE_WINDOWS_OS ) {
+            if ( $1 && $1 ne '64' ) {
+                $self->[UA_OS] = $e;
+                next;
+            }
         }
         push @extras, $e;
     }
@@ -191,7 +198,7 @@ sub _extract_dotnet {
 
 sub _fix_opera {
     my $self = shift;
-    return if ! $self->[UA_EXTRAS];
+    return 1 if ! $self->[UA_EXTRAS];
     my @buf;
     foreach my $e ( @{ $self->[UA_EXTRAS] } ) {
         if ( $e =~ RE_OPERA_MINI ) {
@@ -205,7 +212,7 @@ sub _fix_opera {
     $self->_fix_os_lang;
     $self->_fix_windows_nt('skip_os');
     $self->[UA_EXTRAS] = [ @buf ];
-    return;
+    return 1;
 }
 
 sub _fix_generic {
@@ -257,7 +264,7 @@ sub _parse_maxthon {
 
     $self->[UA_ORIGINAL_VERSION] = $v;
     $self->[UA_ORIGINAL_NAME]    = 'Maxthon';
-    return;
+    return 1;
 }
 
 sub _parse_msie {
@@ -287,14 +294,14 @@ sub _parse_msie {
     }
     $self->[UA_EXTRAS] = [ @buf ];
     $self->[UA_PARSER] = 'msie';
-    return;
+    return 1;
 }
 
 sub _parse_firefox {
     my($self, @args) = @_;
     $self->_parse_mozilla_family( @args );
     $self->[UA_NAME] = 'Firefox';
-    return;
+    return 1;
 }
 
 sub _parse_safari {
@@ -310,7 +317,9 @@ sub _parse_safari {
                             ? pop   @{ $thing }
                             : shift @{ $thing }
                             ;
-    $self->[UA_DEVICE]      = shift @{$thing} if $thing->[0] eq 'iPhone';
+    if ( $thing->[0] && $thing->[0] eq 'iPhone' ) {
+        $self->[UA_DEVICE]  = shift @{$thing};
+    }
     $self->[UA_EXTRAS]      = [ @{$thing}, @others ];
 
     if ( length($self->[UA_OS]) == 1 ) {
@@ -320,7 +329,7 @@ sub _parse_safari {
 
     push @{$self->[UA_EXTRAS]}, @junk if @junk;
 
-    return;
+    return 1;
 }
 
 sub _parse_chrome {
@@ -332,7 +341,7 @@ sub _parse_chrome {
     my($name, $version)      = split RE_SLASH, $chrome;
     $self->[UA_NAME]         = $name;
     $self->[UA_VERSION_RAW]  = $version;
-    return;
+    return 1;
 }
 
 sub _parse_opera_pre {
@@ -342,6 +351,7 @@ sub _parse_opera_pre {
                ? pop @{$thing}
                : 0;
     my($name, $version)     = split RE_SLASH, $moz;
+    return if $name ne 'Opera';
     $self->[UA_NAME]        = $name;
     $self->[UA_VERSION_RAW] = $version;
     my $lang;
@@ -412,7 +422,7 @@ sub _parse_mozilla_family {
     }
 
     $self->[UA_EXTRAS] = [ @{ $thing }, @extras ];
-    return;
+    return 1;
 }
 
 sub _parse_gecko {
@@ -522,7 +532,7 @@ sub _generic_moz_thing {
     my($self, $moz, $t, $extra, $compatible, @others) = @_;
     return if ! @{ $t };
     my($mname, $mversion, @rest) = split RE_CHAR_SLASH_WS, $moz;
-    return if $mname eq 'Mozilla';
+    return if $mname eq 'Mozilla' || $mname eq 'Emacs-W3';
 
     $self->[UA_NAME]        = $mname;
     $self->[UA_VERSION_RAW] = $mversion || ( $mname eq 'Links' ? shift @{$t} : 0 );
@@ -531,7 +541,6 @@ sub _generic_moz_thing {
                    :                                             undef;
     my @extras = (@{$t}, $extra ? @{$extra} : (), @others );
 
-
     $self->_fix_generic(
         \$self->[UA_OS], \$self->[UA_NAME], \$self->[UA_VERSION_RAW], \@extras
     );
@@ -615,10 +624,60 @@ sub _generic_compatible {
     return 1;
 }
 
+sub _parse_emacs {
+    my($self, $moz, $thing, $extra, $compatible, @others) = @_;
+    my @moz = split RE_WHITESPACE, $moz;
+    my $emacs = shift @moz;
+    my($name, $version) = split RE_SLASH, $emacs;
+    $self->[UA_NAME]        = $name;
+    $self->[UA_VERSION_RAW] = $version || 0;
+    $self->[UA_OS]          = shift @{ $thing };
+    my @rest = (  @{ $thing }, @moz );
+    push @rest, @{ $extra } if $extra && ref $extra eq 'ARRAY';
+    push @rest, ( map { split RE_SC_WS, $_ } @others ) if @others;
+    $self->[UA_EXTRAS]      = [ grep { $_ } map { $self->trim( $_ ) } @rest ];
+    $self->[UA_PARSER]      = 'emacs';
+    return 1;
+}
+
+sub _parse_moz_only {
+    my($self, $moz) = @_;
+    my @parts = split RE_WHITESPACE, $moz;
+    my $id = shift @parts;
+    my($name, $version) = split RE_SLASH, $id;
+    if ( $name eq 'Mozilla' && @parts ) {
+        ($name, $version) = split RE_SLASH, shift @parts;
+        return if ! $name || ! $version;
+    }
+    $self->[UA_NAME]        = $name;
+    $self->[UA_VERSION_RAW] = $version || 0;
+    $self->[UA_EXTRAS]      = [ @parts ];
+    $self->[UA_PARSER]      = 'moz_only';
+    return 1;
+}
+
+sub _parse_hotjava {
+    my($self, $moz, $thing, $extra, $compatible, @others) = @_;
+    my $parsable            = shift @{ $thing };
+    my($name, $version)     = split RE_SLASH, $moz;
+    $self->[UA_NAME]        = 'HotJava';
+    $self->[UA_VERSION_RAW] = $version || 0;
+    if ( $parsable ) {
+        my @parts = split m{[\[\]]}xms, $parsable;
+        if ( @parts > 2 ) {
+            @parts = map { $self->trim( $_ ) } @parts;
+            $self->[UA_OS]     = pop @parts;
+            $self->[UA_LANG]   = pop @parts;
+            $self->[UA_EXTRAS] = [ @parts ];
+        }
+    }
+    return 1;
+}
+
 sub _parse_docomo {
     my($self, $moz, $thing, $extra, $compatible, @others) = @_;
     if ( $thing->[0] && index(lc $thing->[0], 'googlebot-mobile') != NO_IMATCH ) {
-        my($name, $version) = split RE_SLASH, shift @{ $thing };
+        my($name, $version)     = split RE_SLASH, shift @{ $thing };
         $self->[UA_NAME]        = $name;
         $self->[UA_VERSION_RAW] = $version;
         $self->[UA_EXTRAS]      = [ @{ $thing } ];
@@ -639,7 +698,7 @@ use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 use constant OPERA_FAKER_EXTRA_SIZE => 4;
 
-$VERSION = '0.17';
+$VERSION = '0.21';
 
 sub _is_opera_pre {
     my($self, $moz) = @_;
@@ -727,6 +786,22 @@ sub _is_strength {
     return;
 }
 
+sub _is_emacs {
+    my($self, $moz) = @_;
+    return index( $moz, 'Emacs-W3/') != NO_IMATCH;
+}
+
+sub _is_moz_only {
+    my($self, $moz, $thing, $extra, $compatible, @others) = @_;
+    return $moz && ! @{ $thing } && ! $extra && ! @others;
+}
+
+sub _is_hotjava {
+    my($self, $moz, $thing, $extra, $compatible, @others) = @_;
+    my @hot = @{ $thing };
+    return @hot == 2 && $hot[1] eq 'Sun';
+}
+
 sub _is_generic_bogus_ie {
     my($self, $extra) = @_;
     return $extra
@@ -743,11 +818,11 @@ use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 use Carp qw( croak );
 
-$VERSION = '0.17';
+$VERSION = '0.21';
 
 sub dumper {
     my($self, @args) = @_;
-    my %opt  = @args % 2 ? () : (
+    my %opt = @args % 2 ? () : (
         type      => 'dumper',
         format    => 'none',
         interpret => 0,
@@ -811,6 +886,7 @@ sub _dumper_dumper {
                         $titles[1],
                         q{-} x $max, q{ } x 2, q{-} x ($max*2);
     require Data::Dumper;
+    my @buf;
     foreach my $id ( @ids ) {
         my $name = $args ? $id->{name} : $id;
         my $val  = $args ? $id->{value} : $self->[ $self->$id() ];
@@ -822,11 +898,14 @@ sub _dumper_dumper {
                     $rv =~ s{ ; }{}xms;
                     $rv eq '[]' ? q{} : $rv;
                 } if $val && ref $val;
-        $buf .= sprintf "%s%s%s\n",
+        push @buf, [
                         $name,
                         (q{ } x (2 + $max - length $name)),
                         defined $val ? $val : q{}
-                        ;
+                    ];
+    }
+    foreach my $row ( sort { lc $a->[0] cmp lc $b->[0] } @buf ) {
+        $buf .= sprintf "%s%s%s\n", @{ $row };
     }
     return $buf;
 }
@@ -837,7 +916,7 @@ use warnings;
 use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.17';
+$VERSION = '0.21';
 
 #TODO: new accessors
 #wap
@@ -891,7 +970,7 @@ use strict;
 use warnings;
 use vars qw( $VERSION );
 
-$VERSION = '0.17';
+$VERSION = '0.21';
 
 use base qw(
     Parse::HTTP::UserAgent::Base::IS
@@ -1002,8 +1081,10 @@ sub _do_parse {
         my $pname  = shift @{ $rv };
         my $method = '_parse_' . $pname;
         my $rvx    = $self->$method( @{ $rv } );
-        $self->[UA_PARSER] = $pname;
-        return $rvx;
+        if ( $rvx ) {
+            $self->[UA_PARSER] = $pname;
+            return $rvx;
+        }
     }
 
     return $self->_extended_probe($m, $t, $e, $c, @o) if $self->[IS_EXTENDED];
@@ -1058,6 +1139,9 @@ sub _extended_probe {
     return if $self->_is_netscape( @args ) && $self->_parse_netscape( @args );
     return if $self->_is_docomo(   @args ) && $self->_parse_docomo(   @args );
     return if $self->_is_generic(  @args );
+    return if $self->_is_emacs(    @args ) && $self->_parse_emacs(    @args );
+    return if $self->_is_moz_only( @args ) && $self->_parse_moz_only( @args );
+    return if $self->_is_hotjava(  @args ) && $self->_parse_hotjava(  @args );
 
     $self->[UA_UNKNOWN] = 1;
     return;
@@ -1070,8 +1154,9 @@ sub _object_ids {
 sub _numify {
     my $self = shift;
     my $v    = shift || return 0;
-    $v    =~ s{
+    $v    =~ s{(
                 pre      |
+                rel      |
                 alpha    |
                 beta     |
                 \-stable |
@@ -1079,7 +1164,14 @@ sub _numify {
                 [ab]\d+  |
                 a\-XXXX  |
                 \+
-                }{}xmsig;
+               )}{}xmsig;
+
+    if ( INSIDE_VERBOSE_TEST ) {
+        if ( $1 ) {
+            Test::More::diag("[DEBUG] _numify: removed '$1' from version string");
+        }
+    }
+
     # Gecko revisions like: "20080915000512" will cause an
     #   integer overflow warning. use bigint?
     local $SIG{__WARN__} = sub {
@@ -1089,7 +1181,27 @@ sub _numify {
     # if version::vpp is used it'll identify 420 as a v-string
     # add a floating point to fool it
     $v .= q{.0} if index($v, q{.}) == NO_IMATCH;
-    my $rv = version->new("$v")->numify;
+    my $rv;
+    eval {
+        $rv = version->new("$v")->numify; 1
+    } or do {
+        my $error = $@ || '[unknown error while parsing version]';
+        if ( INSIDE_UNIT_TEST ) {
+            chomp $error;
+            if ( INSIDE_VERBOSE_TEST ) {
+                Test::More::diag( "[FATAL] _numify: version said: $error" );
+                Test::More::diag(
+                    sprintf "[FATAL] _numify: UA with bogus version (%s) is: %s",
+                                $v, $self->[UA_STRING]
+                );
+                Test::More::diag( '[FATAL] _numify: ' . $self->dumper );
+            }
+            die $error;
+        }
+        else {
+            die $error;
+        }
+    };
     return $rv;
 }
 
@@ -1136,8 +1248,8 @@ generated with an automatic build tool. If you experience problems
 with this version, please install and use the supported standard
 version. This version is B<NOT SUPPORTED>.
 
-This document describes version C<0.17> of C<Parse::HTTP::UserAgent>
-released on C<8 October 2009>.
+This document describes version C<0.21> of C<Parse::HTTP::UserAgent>
+released on C<19 October 2011>.
 
 Quoting L<http://www.webaim.org/blog/user-agent-string-history/>:
 
@@ -1287,12 +1399,12 @@ Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2009 Burak Gursoy. All rights reserved.
+Copyright 2009 - 2011 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.10.0 or, 
+it under the same terms as Perl itself, either Perl version 5.12.3 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
